/**
   RealTimeMessage.java

   Copyright (C) 1998 Geometrek and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix;

import matrix.vrml.*;
import java.io.*;
/**
 *  This class builds and parses real time messages.
 *  Echo tells the Server to forward this msg to the sender also.
 *
 *  @version 1.1
 *  @see matrix.deck.RealTimeHandler
 */
public class RealTimeMessage extends Message {

  final public static int TYPE = 2,
						  NONE    = 25,
                          POP     = 50,
                          FOR     = 75,
                          CLR     = 100;

  public int persist = 25;
  public String tag = null;
  public Field data  = null;
  public boolean echo = true;

  public RealTimeMessage(){
  }

  public RealTimeMessage( int persist, String tag, Field data, boolean echo ){
    this.persist = persist;
    this.tag = tag;
    this.data = data;
    this.echo = echo;
  }

 public RealTimeMessage( String tag, Field data, boolean echo ){
    this.tag = tag;
    this.data = data;
    this.echo = echo;
  }

  public RealTimeMessage( DataInputStream in ) throws IOException {
    read( in );
  }

  public int getType(){
    return TYPE;
  }

  public synchronized void read( DataInputStream in ) throws IOException {
    FieldInputStream fin = new FieldInputStream( in );
    persist = fin.readInt();
    tag = fin.readUTF();
    echo = fin.readBoolean();
    data = fin.readField();
  }

  public synchronized void write( DataOutputStream out ) throws IOException {
    if( data == null )
      throw new IOException("No data in RTMessage");
    if( tag == null )
      throw new IOException("No tag in RTMessage");

    FieldOutputStream fout = new FieldOutputStream( out );
    fout.writeInt( persist );
    fout.writeUTF( tag );
    fout.writeBoolean( echo );
    fout.writeField( data );
  }

  public String toString(){
    return persist+" "+tag+" "+echo+" "+data;
  }
}
