/**
   RealTimeSave.java

   Copyright (C) 1998 Geometrek and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



package matrix;

import matrix.vrml.*;
import java.io.*;
/**
 *  This class builds and parses real time messages.
 *  Echo tells the Server to forward this msg to the sender also.
 *
 *  @version 1.1
 *  @see matrix.deck.RealTimeHandler
 */
public class RealTimeSave extends Message {

  final public static int TYPE = 3;

  public String tag = null;
  public Field data  = null;
  public Field state = null;

  public RealTimeSave(){
  }

  public RealTimeSave( String tag, Field data, Field state ){
    this.tag = tag;
    this.data = data;
    this.state = state;
  }

  public RealTimeSave( DataInputStream in ) throws IOException {
    read( in );
  }

  public int getType(){
    return TYPE;
  }

  public synchronized void read( DataInputStream in ) throws IOException {
    FieldInputStream fin = new FieldInputStream( in );
    tag = fin.readUTF();
    state = fin.readField();
    data = fin.readField();
  }

  public synchronized void write( DataOutputStream out ) throws IOException {
    if( data == null )
      throw new IOException("No data in RTSave");
    if( tag == null )
      throw new IOException("No tag in RTSave");
   if( state == null )
      throw new IOException("No state in RTSave");
    FieldOutputStream fout = new FieldOutputStream( out );
    fout.writeUTF( tag );
    fout.writeField( state );
    fout.writeField( data );
  }

  public String toString(){
    return tag+" "+state+" "+data;
  }
}
