/*
   RoomControl.java

Copyright (C) 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 

package matrix;

import matrix.vrml.*;
import java.io.*;
/**
 *  This class builds and parses messages listing user's, object's and room names and number of room occupants.
 *  
 *
 *  @version 1.1
 *  @see matrix.deck.FrontEnd and its inner class WWWBuffer.
 */
public class RoomControl extends Message {
  final public static int SET     = 100,
                          GET     = 300,
                          ADDMEM  = 400,
                          REMMEM  = 500,
						  PASSWD  = 600,
                          TYPE = 6;

  public int command;
  public String name;
  public int size;
  public String [] rc;

  public RoomControl(){
  }

  public RoomControl( int command, String name, String [] arg ) throws IOException {
   if( name == null || arg == null )
      throw new IOException("Data items are null");
     this.command = command;
    this.name = name;
    this.size = arg.length;
    rc = arg;
  }

  public RoomControl( DataInputStream in ) throws IOException {
    read( in );
  }

  public int getType(){
    return TYPE;
  }

  public synchronized void read( DataInputStream in ) throws IOException {
    command = in.readInt();
    name = in.readUTF();
    size = in.readInt();
  rc = new String[size];
    for (int i = 0; i < rc.length; i++) {
      rc[i] = in.readUTF();
    }
  }

  public synchronized void write( DataOutputStream out ) throws IOException {
   if( name == null || rc == null )
      throw new IOException("Data items are null");
    out.writeInt( command );
    out.writeUTF( name );
    out.writeInt( size );
  for (int i = 0; i < rc.length; i++) {
     out.writeUTF( rc[i] );
    }
    
  }

  public String toString(){ 
String  st = command+" "+name+" "+size+" ";
for (int i = 0; i < rc.length; i++) {
  st += rc[i] + " ";
    }
return st;
}

}