/**
  TCPConnection.java

Copyright (C) 1997,1998 1999 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 

package matrix;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * This class encapsultes a Socket and the In/Outputstreams used for communication.
 * It gets or sends Messages and is used in both the client and
 * the server program.
 *
 * @see Message
 * @version 1.1
 */
public class TCPConnectionApplet extends Connection {
  private Socket ConSocket = null;
  private DataInputStream in;
  private DataOutputStream out;
  private boolean alive;
  private int buffer = 2048;

 public TCPConnectionApplet() {
    alive = false;
  }

  public TCPConnectionApplet( int buffer ){
    this.buffer = buffer;
    alive = false;
  }

  public TCPConnectionApplet( Socket socks ) throws IOException {
    open( socks );
  }

  public TCPConnectionApplet( Socket socks, int buffer ) throws IOException {
    this.buffer = buffer;
    open( socks );
  }

  public TCPConnectionApplet( String host, int port ) throws IOException {
    open( host, port );
  }

  public TCPConnectionApplet( String host, int port, int buffer ) throws IOException {
    this.buffer = buffer;
    open( host, port );
  }

  public void open( Socket socks ) throws IOException {
    close();
    ConSocket = socks;
    in = new DataInputStream( new BufferedInputStream( ConSocket.getInputStream(), buffer) );
    out = new DataOutputStream( new BufferedOutputStream( ConSocket.getOutputStream(), buffer) );
    alive = true;
  }

  public void open( String host, int port ) throws IOException {
    open( new Socket( host, port ));
  }

   /**
     *  New 11-12-05 startMsgBuffer delays the start of the thread of the inner MessageBuffer class
     *  Delay necessary for Mac Cosmo Beta. Called by connect method in the Backend class.
     */

    public void startMsgBuffer(){
  }

  public void close() {
    alive = false;
    try {
      if( ConSocket != null ) {
        in.close();
        out.close();
        ConSocket.close();
      }
    }
    catch(IOException e) {};
    in = null;
    out = null;
    ConSocket = null;
  }

  protected void sendRawMessage( Message msg ) throws IOException {
    synchronized( out ){
      out.writeInt( msg.getType());
      msg.write(out);
      out.flush();
    }
  }

  public void sendLoginMessage( Message msg ) throws IOException {
    }

  public void sendMessage( Message msg ) throws IOException {
      sendRawMessage( msg );
  }

    public void sendMessage2( Message msg ) throws IOException {
  }

    public void sendMessage3( Message msg ) throws IOException {
  }

  public Message getMessage() throws IOException {
    Message msg;
    synchronized( in ){
      int type = in.readInt();
      switch( type ){
        case ControlMessage.TYPE :
          msg = new ControlMessage( in );
          break;
        case RealTimeMessage.TYPE :
          msg = new RealTimeMessage( in );
          break;
		case RealTimeSave.TYPE :
          msg = new RealTimeSave( in );
          break;
		case WhoWhatWhere.TYPE :
          msg = new WhoWhatWhere( in );
          break;
		 case AdminControl.TYPE :
          msg = new AdminControl( in );
          break;
		 case RoomControl.TYPE :
          msg = new RoomControl( in );
          break;
		 case ObjectEnvelope.TYPE :
		  msg = new ObjectEnvelope( in );
          break;
		 case TriggerEnvelope.TYPE :
          msg = new TriggerEnvelope( in );
          break;
		 case ControlEvent.TYPE :
          msg = new ControlEvent( in );
          break;
        default :
          throw new UnknownMessageException("unknown message type "+type);
      }
    }
    return msg;
  }

 public void exchange( ObjectInputStream in, ObjectOutputStream out ) throws Exception {
   }

  public boolean isAlive() { return alive; }

  public InetAddress getInetAddress(){
    if(isAlive())
      return ConSocket.getInetAddress();
    else
      return null;
  }

    public void setSoTimeout( int iSTO ) throws SocketException {
    ConSocket.setSoTimeout( iSTO );
    } 

  public int getPort(){
    if(isAlive())
      return ConSocket.getPort();
    else
      return -1;
  }

  public void setShutDown() throws IOException, InterruptedException {
  }

}
