/*
  RealTimeHandler.java

  Copyright (C) 1998 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.deck;
import matrix.RealTimeMessage;
import matrix.vrml.*;
import java.util.*;
import java.io.*;
import java.net.*;
import vrml.eai.*;
import vrml.eai.field.*;


/**
 *  sends and receives real time messages and manages RealTimeConsumers
 * ( for example Routes and Avatars )
 *
 *  @version 1.1
 *  @see matrix.RealTimeMessage
 *  @see matrix.deck.eai.Route
 *  @see matrix.deck.eai.Avatar
 */
public class RealTimeHandler implements RealTimeInfo, Runnable {
    private boolean updateOk = false;
    private boolean suicide = false;
    private Hashtable consumers;
    private Vector msgs;
    private float [] fUserPosition = new float [] { 0.0f, 0.0f, 0.0f };
    private Thread runner  = null;
    private int count = 0, lost = 0, iUserPos = -1, iCheckLoc = 0;
    private Viewer viewer;
    private Vector vAvatars;
    private FakeProxy FP;

    public RealTimeHandler() throws IOException {
	consumers = new Hashtable();
    vAvatars = new Vector();
  msgs = new Vector();
	runner = new Thread( this, "handler");
	runner.start();
    }

  public void setViewer( Viewer viewer ){
        this.viewer = viewer;
    }
   
  void start(){ 

  }
  
  /**
    *  Thread delivers VRML info from synchronized msgs Vector to the appropiate consumer by name 
    */
    public void run(){
    FP = new FakeProxy( this, 400 );
	RealTimeConsumer help;
	RealTimeMessage recMsg;
    holdRTH();
    while( Thread.currentThread() == runner ){
        try{
            if( !msgs.isEmpty()){
                if( viewer != null&&updateOk==true){ 
                     viewer.beginUpdate();
                } 
                while( !msgs.isEmpty()){
                    synchronized(msgs){
                        recMsg =(RealTimeMessage) msgs.firstElement();
                        msgs.removeElementAt(0);
                    }
                    help =(RealTimeConsumer) consumers.get(recMsg.tag);
                    if( help != null )
                        help.handleMessage( recMsg.tag, recMsg.data );
                }
                if( viewer != null&&updateOk==true ){
                    viewer.endUpdate();
                }
            } else {  
                synchronized(msgs){
                   if(suicide==false){
                    msgs.wait();
                } 
                  else { 
                        runner = null;
                       }
                    }
                }
            } 
        catch(InterruptedException e){}
        catch(Exception e){
            System.err.println("RTHandler "+e);
        }
    }
    }
    /**
    * delivers the VRML information from network to the msgs Vector
    */
    public void handle( RealTimeMessage msg ){
	synchronized( msgs ){
	    if( !msgs.isEmpty()){
		Enumeration list = msgs.elements();
		RealTimeMessage help;
		while( list.hasMoreElements()) {
		    help = (RealTimeMessage) list.nextElement();
		    if( help.tag.equals( msg.tag ) && help.data.getType() == msg.data.getType()){
			msgs.removeElement( help );
			lost ++;
			break;
		    }
		}
	    }
	    msgs.addElement( msg );
	    msgs.notify();
	}
	count ++;
	if( count == 100 ){
	    //	    System.out.println("rt lost "+lost+" %");
	    count = 0;
	    lost = 0;
	}
    }
   /**
    *  Adds new VRML Avatar user or World.
    */
    public synchronized void addConsumer( RealTimeConsumer customer ){
	if(consumers.containsKey(customer.getName())){
	    System.err.println("RTHandler "+customer.getName()+" exists allready");
	    return;
	}
	consumers.put( customer.getName(), customer );
    if(customer.getbAvatar()){ vAvatars.addElement( customer );
    System.out.println( "RTH addConsumer bAvatar is true" );
    if(vAvatars.size()==1){ FP.setInterrupt();}
         }
     else {  System.out.println( "RTH addConsumer bAvatar is false" ); }
    }

   /**
    * Gets VRML Avatar user or World.
    */
    public RealTimeConsumer getConsumer( String name ){
	return (RealTimeConsumer)consumers.get( name );
    }

    /**
    * Removes VRML Avatar or World. 
    */

    public synchronized void removeConsumer( String name ){
	RealTimeConsumer help = (RealTimeConsumer) consumers.get(name);
	if( help != null ){
	    help.delete();
	    consumers.remove(name);
    if(help.getbAvatar()){ vAvatars.removeElement( help ); }
	}
    }

    public Enumeration getConsumers(){
	return consumers.elements();
    }
    /**
    * New 11-12-05 Sets whether the begin and end update methods exist for the VRML plugin
    * Was necessary to for the combination of Internet Explorer and Cosmoplayer. However that 
    * combination has a tendency to lock up so quickly, it is no longer supported.
    */

   public void rth_updateOk( boolean tf ){
  updateOk = tf;
    }

   /**
    *   updates all users
    */
    public void update(){
	Enumeration list = getConsumers();
	while( list.hasMoreElements())
	    ((RealTimeConsumer)list.nextElement()).update();
    }
    /**
    * updates specific user by name
    */
    public void update( String name ){
	RealTimeConsumer help = (RealTimeConsumer) consumers.get(name);
	if( help != null ){
	    help.update();
	}
    }

  
    /**
    * New 11-12-05 Finds name of target drone av and delivers request for beamto coordinates.
    */

  public void findbeamto( String target ){ 
  RealTimeConsumer help = (RealTimeConsumer) consumers.get(target);
 	if( help != null ){
	    help.beamto( target );
	}
    }

   public void setUserView( String target ){
    RealTimeConsumer help = (RealTimeConsumer) consumers.get(target);
 	if( help != null ){
	    help.userView( target );
	}
 }

   public void setObjectRoutes( String target ){
    RealTimeConsumer help = (RealTimeConsumer) consumers.get(target);
 	if( help != null ){
//System.out.println("RealTimeHandler setObjectRoutes2 got target: " + target); 
	    help.objectRoutes( target );
	}
 }

   public void checkLocations(){ 
   boolean bUPseen = false;
 if(!vAvatars.isEmpty()){
 if(iUserPos==iCheckLoc){ bUPseen = true; }
   Object [] aRTC = vAvatars.toArray();
   for( int i = 0; i<aRTC.length; i += 1 ){
      ((RealTimeConsumer)aRTC[i]).checkPosition( fUserPosition, bUPseen );
	  }
   iCheckLoc = iUserPos;
  }
 else { if(FP!=null){ FP.setSleepy( -1 ); } }
   }

   public void setUserPosition( float [] flt ){
    fUserPosition = flt;
  if(iUserPos<100){  iUserPos += 1;	} else { iUserPos = 0; }
  }




  /**
    * clears all consumers
    */

    public synchronized void clear(){
    //Using Sun JVM now!
	// appearantly the MS implementation of HashTable treats
	// elements in a different order or depends on elements
	// not being removed before they are accessed through the
	// enumeration. Therefore we have to reinit the enumeration
	// everytime, in case the removal removes other consumers too.


	Enumeration list = consumers.keys();
	while( list.hasMoreElements()){
	    String help = (String)list.nextElement();
	    removeConsumer(help);
	}
	consumers.clear();
    vAvatars.clear();
    }

   public synchronized void clear2(){
	consumers.clear();
	vAvatars.clear();
    }


  void holdRTH(){
   synchronized( msgs ){
  suicide = false;
 try {
  msgs.wait();
   }
catch(InterruptedException e ){}
      }
  }
 
 /**
  *  closes, nullifies runner
  *  New 11-12-05 Notifying the synchronized classes of threads tends to cause crashes
  *  in Cosmoplayer, so those calls have been removed. 
  *  See RealTimeHandler close method.
  */

    public void close()  {
  if(FP!=null){ FP.stop(); }
  if(runner!=null){
  synchronized( msgs ){
  suicide = true;
  msgs.removeAllElements();
  msgs.notify();
      }
     }
	  }


 }      