/*
  Viewer.java

  Copyright (c) 2000 Geometrek
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.deck;

/**
 * The Viewer represents the interface to work with any Viewer. Current
 * implementations include a Shout3D Viewer and an EAI / VRML97 viewer.
 *
 * @version 1.0
 */
public interface Viewer {

	//public void viewLoaderWait();

    /** loads the world as background. Will also return a SharedObject but
   * might need some more initialization.
   * @param name a name given to identify the world object
   * @param url location of the VRML file holding the world content
   */
    

    public void loadWorld( String name, String url );

    /** loads a SharedObject into the simulation. Identified by a name.
     * @param name a name given to identify the object
     * @param url location of the VRML file holding the objects content
     */
    public void addSharedObject( String name, String url );

    /** removes the SharedObject identified by name.
     * @param name name of the object to identify it
     */
    public void removeSharedObject( String name );

    public void removeSharedObject2( SO object ); 

    /** adds a ViewerListener for callbacks on successful loading.
     * @param listener the listener to add to the Viewer object
     */
    public void addViewerListener( ViewerListener listener );

    /** removes a ViewerListener
     * @param listener the listener to remove from the Viewer object
     */
    public void removeViewerListener( ViewerListener listener );
    
    /**
     * begins a critical section, where no rendering occurs, to
     * update several events at once. This is only supported in
     * newer VRML browsers like Contact or Cortona.
     */    
    public void beginUpdate();
    
    
    /**
     * ends the critical section.
     * @see beginUpdate
     */
    public void endUpdate();

}
