/**
  CtrlRoute.java

 Copyright (C) 1998 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.deck.eai2;

import matrix.RealTimeMessage;
import matrix.vrml.*;
import matrix.vrml.eai2.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.CtrlEvent;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import java.io.*;
import vrml.eai.*;
import vrml.eai.field.*;
import vrml.eai.event.*;

/**
 * Controls a single network state with a users name. Hence a specialized Route
 *
 * @version 1.1
 */
public class CtrlRoute implements CtrlEvent, RealTimeConsumer,  VrmlEventListener, Runnable {
    private String name;
    private String username = "";
    private String sLock = "#open";
    private Node myNode;
    private boolean continous = false, valChanged = false, echo = false, dead = false, lock = false;
    private EventOut callingEvent;
    private EventIn fromnet;
    private Network postOffice;
    private long lTime = 0;
    private long lSleep = 400;
	private int stor = 25;
    private int iVn = 0;
    private Field val= null, lastVal = null;
    private int type;
	private EventInSFString setName = null;
    private EventInSFString setLock = null;
    private CtrlName ctrlname;
    private Thread runner = null;
	public boolean bAvatar = false;


	public class CtrlName implements VrmlEventListener {
 private EventOutSFTime time_out = null;

    public CtrlName( EventOutSFTime time_changed ) throws Exception {
	 time_out = time_changed;
 time_out.addVrmlEventListener(this);
     }

	public void eventOutChanged( VrmlEvent vrmlevent ) {
   
       BaseField bf =  vrmlevent.getSource();
		if( bf.getType()== BaseField.SFTime ) {
    try {
     setName.setValue(username);
	 if(sLock.equals("#open")){
     postOffice.sendControlEvent( name, new String [] {username, "#lock"} );
     }
     else 
	 if(sLock.equals(username)&&!username.equals("")){
     postOffice.sendControlEvent( name, new String [] {username,"#open"} );
       }
     }  
     catch(IOException e){System.err.println("CtrlRoute: " + e); }
     catch(InterruptedException f){}
    }
 else
    return;
   }

public void delete(){                              
if(time_out!=null){ time_out.removeVrmlEventListener(this);	
 time_out = null;
    }
   }

}

    
    public CtrlRoute( Node netnode, Network office, int VN, long slp, String user ) throws RuntimeException {
	postOffice = office;
	myNode = netnode;
	iVn = VN;
    lSleep = slp;
    username = user;
    try {
	name = (((EventOutSFString)myNode.getEventOut("tag")).getValue() );
	if(name.endsWith("_P")){
	stor = RealTimeMessage.POP;
     }
    if(name.endsWith("_F")){
    stor = RealTimeMessage.FOR;
    }
    if(name.endsWith("_C")){
    stor = RealTimeMessage.CLR;
    }
    //System.out.println("CtrlRoute name-tag: " + this.name );
	continous = ((EventOutSFBool)myNode.getEventOut("cont")).getValue();
	echo = ((EventOutSFBool)myNode.getEventOut("echo")).getValue();
    ctrlname = new CtrlName( ((EventOutSFTime)myNode.getEventOut("control_time"))   );
    callingEvent = myNode.getEventOut("value_tonet");  // GC workaround
    callingEvent.addVrmlEventListener(this);
    setLock = ((EventInSFString)myNode.getEventIn("set_lock"));
	setName = ((EventInSFString)myNode.getEventIn("set_name"));
    setName.setValue( username );
	fromnet = myNode.getEventIn("value_fromnet");
	type = (callingEvent.getType());
     }
    catch(NullPointerException f){ System.out.println( "CtrlRoute error: " + f ); }
    catch(InvalidEventInException g){System.out.println( "CtrlRoute error: " + g );  }
    catch(InvalidEventOutException h){System.out.println( "CtrlRoute error: " + h ); } 
    catch(Exception k){System.out.println( "CtrlRoute error: " + k ); }
	try {
	    lastVal = EAITool.makeField( callingEvent );
	}
	catch(MatrixInvalidFieldException e){
	    System.err.println("Route "+name+" exeption in constructor "+e);
	}
	if( continous == true ){
	    runner = new Thread( this, name );
	    runner.start();
	}
    }
    
    public void setName( String newname ){
	this.name = newname;
	// System.out.println("CtrlRoute new name-tag: " + name);
    }

    public String getName(){
	return name;
    }
    
     

    public void handleMessage( String tag, Field data ) throws ConsumerException {
	if(!tag.equals(name))
	    throw new ConsumerException(name+" matches not "+tag);
  
	try {
	    EAITool.setEventIn( fromnet, data );
	    lastVal = data;
	}
	catch(Exception e){
	    throw new ConsumerException(name+"\n\t"+e);
	}
    }

    public void handleLock( String [] lock ){
        if(lock[0]!=null){ sLock = lock[0];
    if(setLock!=null){ setLock.setValue( sLock ); }
     }
    }

    public void setCtrlName( String name ){
	this.name = name;
    }

	 public String ctrlName(){
    return name;
    }   

	public void ctrlDelete(){
     }

public void userView( String target ){
}

public void objectRoutes( String target ){
 }

  public void setUserPosition( float[] flt ){
  }

  public boolean getbAvatar(){
  return bAvatar;
     }

   public void checkPosition( float[] fUP, boolean bUPs  ){
 }

 public void beamto( String target ){
}

    public void update(){
    }

    private synchronized void set_valChanged( boolean tf ){
    valChanged = tf;
    }

    public void run(){
	while( Thread.currentThread() == runner ) {
	    try{
		if( valChanged == true ){
		    postOffice.sendRealTime( stor, name, val, echo );
		    lastVal = val;
	         set_valChanged ( false );
		}
		Thread.sleep(lSleep);
	    }
	    catch(InterruptedException e){}
	    catch(IOException e){
		System.err.println("IOEx in route "+name+".run\n\t"+e);
	    }
	}
    }

    public void eventOutChanged(VrmlEvent vrmlevent) {
  if(!sLock.equals("")&&sLock.equals(username)){
	if( dead == true ) return;
 BaseField bf =  vrmlevent.getSource();
 try {
	    if( type != bf.getType() ) {
		System.err.println("Got wrong value from "+name);
		return;
	    }
	    val = EAITool.makeField( callingEvent );
	
	    if( continous == false){
		postOffice.sendRealTime( stor, name, val, echo );
		lastVal = val;
		set_valChanged ( false ); 
	    }
	    else set_valChanged ( true );
	}
    catch(IOException f){
    System.err.println("CtrlRoute: " + f); return;
   }
	catch(Exception e) {
	    System.err.println("Error converting msg from "+bf+":"+e); return;
	}
 // lTime = lTime2; }
     }
    }

    public void delete() throws RuntimeException {
	runner = null; 
if(callingEvent!=null){ callingEvent.removeVrmlEventListener(this); }
if(ctrlname!=null){ ctrlname.delete(); }
 fromnet = null; 
 val = lastVal = null;
 myNode = null;
 callingEvent = null;
 dead = true;
    }
}
