/**
  UserAvatar.java

 Copyright (C) 1998 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 

package matrix.deck.eai2;

import matrix.RealTimeMessage;
import matrix.vrml.*;
import matrix.vrml.eai2.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import matrix.deck.UserInterfaceListener;
import java.io.*;
import java.util.*;
import vrml.eai.*;
import vrml.eai.field.*;
import vrml.eai.event.*;


/**
 *  manages Users avatar in local world, displays it in Ghost mode and sends updates to drones.
 *  also manages BOMU behaviours.
 *
 *  walking behaviour automation by Andrew Gerngross <agern@ix.netcom.com>.
 *
 *  @version 1.3
 */
public class UserAvatar implements  RealTimeConsumer, Runnable { 
   /**
    * New 11-12-05 Node for Chat
    */
    private EventInMFString yak;
    /**
    * New 11-12-05 String variables for keep track of avatar features
    */
    private String talk = "nah", human = "hah", inMotionBehaviour = null, notInMotionBehaviour = null;
    private long lSleep = 400, lDefault = 400;
    /**
    * New 11-12-05 a is used as a boolean to determine if chat text is to be displayed.
    */
    private boolean bChat = false, bOR = true, bP = true, bO = true, bReset = true, moving = false; 
    /**
    * New 11-12-05 vn is viewernumber from EAIViewer class for which VRML plugin determination.
    */
    private int vn = 0, iO = -1, iP = -1, iActP = 0, iActR = 0, iRest = 0, iReset = 0;
     /**
    * New 11-12-05 newvp is for a Viewpoint that will bind when the user leaves ghost mode so the user wont go back to world entry viewpoint!
    * Not necessary in cosmoplayer!
    */
    private String newvp = "";
     /**
     * name is user' name. New 11-12-05 name_text is for users name display to be create VRML from String
     */
    private String name, name_text;
   /**
    * New 11-12-05 Beta is added as variable for beamto
    */
    private Beta avdeck;
  
    /**
     * Network for communicating 
     */ 
    private Network postOffice;
    /**
     *  Ghost mode state
     */
    private boolean show = false;
    private boolean bHUD = false;
    private EventInSFBool setProxy = null;
    private EventInSFBool setUserAvatar = null;
    private float Zaxis = (float)1.85;
	private float [] beam = new float[7];
   /**
    * New 11-12-05 position with beam to information
    */ 
    private float [] posflt = new float[3];
	/**
    * New 11-12-05 rotation with beam to information
    */ 
    private float [] rotflt = new float[4];
   /**
    * New 11-12-05 beamtostate needed to juggle which viewpoints to bind when leaving ghost mode. See hide and show methods  
    */
    public boolean beamtostate = false;
    public boolean bAvOk = true;
    /**
     * proxy holds the ProximitySensor from WorldObject class to get user location info.
     * New 11-12-05 vp holds the new Viewpoint to retain  where user's avatar stopped when user went into ghost mode. Again, not needed in Cosmoplayer
     */
    private Node proxy, anchor, avatar = null, vp = null;
    /**
     * New 11-12-05 fov holds variable vp's fieldOfView
     */
    private EventInSFFloat fov;
   /**
    * New 11-12-05 bind and bind2 are for viewpoints coordinated with boolean beamtostate. See hide and show methods
    */
    private EventInSFBool setGhostHUD = null;
  
    private EventInSFBool bind, bind2;
   /**
     * pos is the Avatar's translation exposedField
     *
     * New 11-12-05 vppos holds Viewpoint variable vp's position for changing when leaving ghost mode
     */ 
    private EventInSFVec3f pos, vppos, ghostVpPos;
    /**
    * New 11-12-05 eventIn where all the position and rotation info of beamto target avatar is given to the users av
    */
    private EventInMFFloat mffloat;
    /**
     * rot is the Avatar's rotation exposedField
     *
     * New 11-12-05 vpori holds Viewpoint variable vp's orientation for changing when leaving ghost mode
     */ 
    private EventInSFRotation rot, vpori, ghostVpRot;
    /**
     * Ghost mode Switch
     */
    private EventInSFInt32 which;
    private EventInSFString set_name = null;
    /**
     *  offset is for determining UserAvatars position in reference to ground
    */
    private SFVec3f offset = new SFVec3f((float)0.0,(float)-1.85,(float)0.0);
    private SFVec3f ghostset = new SFVec3f((float)0.0,(float)1.6,(float)5.0); 
    /**
     * posVal is the current position from the ProximitySensor with the offset calculated in
     *
     * New 11-12-05 posVal2 holds the current position when going into ghost mode. It is given to vppos when leaving ghost mode. Not used in Cosmoplayer!
     */ 
    private SFVec3f posVal, posVal2;
    /**
     * rotVal is current orientation from the ProximitySensor.
     *
     * New 11-12-05 rotVal2 holds the current orientation when going into ghost mode. It is given to vppos when leaving ghost mode. Not used in Cosmoplayer!
     */ 
    private SFRotation rotVal, rotVal2;

	private EventInSFBool [] gestures = null;
   /**
    * holds Routes for found Shared Event Protos
    */
    private Vector states = new Vector();

     /**
      * posChanged and rotChanged are used by the thread to determine if new UserAvatar location updates should be sent to network
      * dead means user is dead and updates are not to be done.
      */
    private boolean  dead = false; 
    private Thread runner = null;
     /**
      * holds the last time the Avatar moved
      */
    private java.util.Date lastMoved = null;  
	private Rot_PosObserver ropo = null;
	private WorldSendChat WSC = null;
    private WorldSendLocalChat WSLC = null; 
	public boolean bAvatar = false, bdmStudioAvatar = false;



    /**
     * builds a new UserAvatar. It parses the loaded nodes and construct the needed scenegraph
     * and gets the events etc. Also builds all the necessary Route objects, if it is
     * a SharedObject.
     * New 11-12-05 the gesture list is now also used to list UserAvatar options 
     * (NG_chat_text,NG_humanoid,NG_no_lod) "NG_" prefix is for the UserAvatar
     * so the option does not show up as a gesture. 
     * NG_chat_text means the avatar is setup for chat text display.
     * NG_humanoid the avatar has Deep Matrix's humanoid animation methods: "inMotionBehaviour" 
     * and "notInMotionBehaviour"
     * NG_no_lod means the avatar does not want the built in LOD.
     *
     * One of the stranger aspects of UserAvatar is the use of EventInMFString wc in the constructor
     * This is a VRML eventin from the World class to display chat strings as VRML text. Moving this
     * eventIn World.class -> UserAvatar -> Beta  stops Cosmoplayer from crashing. Don't ask why just accept :)
     *
     * user_av_finished() is called from Beta when the user's avatar is finished.
     * drone_av_finished( name, false ) is called from Beta if the user's avatar throws an error.
     *
     * @param object the loaded object, containing the name and a Transform
     * @param browser reference to the VRML browser
     * @param postOffice reference to the Network object
     *
     
     */
    public UserAvatar( SO2 object, Network post, Browser browser, Rot_PosObserver RoPo, Node prox, Beta deck, int VIN, float diff, long slp )
	throws RuntimeException {
 // System.out.println("UserAvatar started");
 Node [] Foo = null;
 Node [] xform = null;
 Node [] nodes = null;
 boolean bTest = false;
 boolean bGest = false;
 EventOut EOfield = null;
 EventIn EIfield = null;
 this.ropo = RoPo;
 Zaxis = diff;
	name = object.getName();
	postOffice = post;
	proxy = prox;
 avdeck = deck;
 vn = VIN;
 lDefault = slp;
 lSleep = lDefault;
 //offset = new SFVec3f(0,-diff,0);
   try {
 bind = (EventInSFBool)proxy.getEventIn("set_bind");
}
catch(InvalidEventInException e){ System.out.println("No set_bind: "+ e); 
 avdeck.drone_av_finished( name, false );
	    return;
}
catch(NullPointerException f){ System.out.println("No set_bind: "+ f); 
    avdeck.drone_av_finished( name, false );
	    return;
     }
try {
 mffloat = (EventInMFFloat)proxy.getEventIn("setBeamTo");
}
catch(InvalidEventInException e){ System.out.println("No setBeamTo: "+ e); 
   avdeck.drone_av_finished( name, false );
	    return;
    }
catch(NullPointerException f){ System.out.println("No setBeamto: "+ f); 
    avdeck.drone_av_finished( name, false );
	    return;
    }

      
 name_text = ( "Transform { " + 
			"translation 0 1.75 0 " +
			"children [ " + 
             "LOD {  " +
             "range 25 " +                                         
             "level [ " +
              "Group { " +
              "children [ " +
			  "Billboard {   " +
              "axisOfRotation 0 0 0 " +
			  "children [  " +
			  "Shape { geometry Text { string [\"" + name.substring(1) + "\"] maxExtent 4  " +
			  "fontStyle FontStyle { family [\"SANS\"] justify \"MIDDLE\" size .25 } } " +
			  "} " +
             "]}]} " +
             "Group {} " +                                          
						       "]}]}  " );
	// get nodes
	try {
 xform = object.getTransforms2();
    }
  catch(NullPointerException np){ System.out.println( "object.getTransforms(): " + np );
   avdeck.drone_av_finished( name, false );
	    return;}
if(xform[0].getType().equals("Avatar2")){
 System.out.println("UserAvatar Avatar2");
  	try {
 Foo = object.getTransforms1();
 nodes = ((EventOutMFNode)Foo[0].getEventOut("children_changed")).getValue();
    }
  catch(NullPointerException np){ System.out.println( "object.getTransform(): " + np );
  avdeck.drone_av_finished( name, false );
	    return; 
     }
	// check for SharedObject definition
  for( int i = 0; i < nodes.length; i++ ) {
    if(!deck.bLogOut){
    System.out.println("UA Node Type is: "+ nodes[i].getType());
	if( nodes[i].getType().equals("SharedObject")){
      System.out.println("Avatar rejected! SharedEvents for User Avatars are not used this version!");
	    avdeck.drone_av_finished( name, false );
	    break;
   }
//	    if(!name.endsWith("_Obj")){ name = name + "_Obj" ;}
//	     
   
if( nodes[i].getType().equals("Avatar")){
System.out.println( "Is Avatar" );
 EventOutSFBool eosfb = null;
try {
eosfb = (EventOutSFBool)nodes[i].getEventOut("isAvatar"); 
 }
catch(InvalidEventOutException f){
  System.out.println(name + " is not a bxx avatar! GOOD continue!");
 }
if(eosfb!=null){ 
   avdeck.drone_av_finished( name, false );
    return;
 }
avatar = nodes[i];
if(avatar!=null){ 
EventOutMFNode nsc = null;
//  try {
// EIfield = nodes[i].getEventOut("states_changed");
// nsc = (EventOutMFNode)nodes[i].getEventOut("states_changed");
//  }
// catch(NullPointerException e){ 
// System.out.println( "UserAvatar doesn't have states_changed. ");
// }
// catch(InvalidEventOutException f){ 
// System.out.println( "UserAvatar doesn't have states_changed. ");
// }
// catch(InvalidVrmlException g){ 
// System.out.println( "UserAvatar states_changed error: " + g);
// 	 avatar = null;
//      avdeck.drone_av_finished( name, false );
//      return;
// }
// catch(Exception h){
// System.out.println( "UserAvatar states_changed error: " + h);
// 	avatar = null;
//     avdeck.drone_av_finished( name, false );
//     return;
// }
try {
EOfield = null;
bTest = false; 
EOfield = nodes[i].getEventOut("to_network_chat");
if(EOfield!=null&&EOfield.getType() == BaseField.MFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
System.out.println( "UserAvatar to_network_chat error: " + e);
    avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "UserAvatar doesn't have  to_network_chat.");
}
catch(InvalidVrmlException j){
 System.out.println( "UserAvatar to_network_chat error: " + j);
    avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar to_network_chat error: " + h);
	avatar = null;
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){   
try {
EventOutMFString eomfs = (EventOutMFString)EOfield;
WSC  = new WorldSendChat( deck, eomfs );
System.out.println( "UserAvatar has to_network_chat.");
  }
catch(NullPointerException e){ 
System.out.println( "UserAvatar doesn't have  to_network_chat.");
    avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "UserAvatar doesn't have  to_network_chat.");
	avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidVrmlException j){
 System.out.println( "UserAvatar to_network_chat error: " + j);
    avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar to_network_chat error: " + h);
	avatar = null;
    avdeck.drone_av_finished( name, false );  
    return;
}
}

try{
 EOfield = null;
 bTest = false; 
 EOfield = nodes[i].getEventOut("to_local_chat");
if(EOfield!=null&&EOfield.getType() == BaseField.MFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
  System.out.println( "UserAvatar to_local_chat error: " + e);
    avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "UserAvatar doesn't have  to_local_chat.");
}
catch(InvalidVrmlException j){
 System.out.println( "UserAvatar to_local_chat error: " + j);
    avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar to_local_chat error: " + h);
	avatar = null;
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){
try {
EventOutMFString eomfs = (EventOutMFString)nodes[i].getEventOut("to_local_chat");
WSLC  = new WorldSendLocalChat( deck, eomfs );
System.out.println( "UserAvatar has  to_local_chat.");
  }
catch(NullPointerException e){ 
 System.out.println( "UserAvatar to_local_chat error: " + e);
  	avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "UserAvatar to_local_chat error: " + f);
    avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidVrmlException j){
 System.out.println( "UserAvatar to_local_chat error: " + j);
	avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar to_local_chat error: " + h);
	avatar = null;
    avdeck.drone_av_finished( name, false ); 
    return;
 }
}
try {
EOfield = null;
bTest = false;
EOfield = nodes[i].getEventOut("gestures");
if(EOfield!=null&&EOfield.getType()==BaseField.MFString){
 bTest = true;
 }
}
  catch(InvalidEventOutException e){
     System.out.println(name+" does not have gestures!");
	}
  catch(NullPointerException f){ 
    System.out.println(name+" does not have gestures!" );
     }
  catch(InvalidVrmlException g){ 
    System.out.println(name+" gestures errors: " + g );
    avdeck.drone_av_finished( name, false );
    avatar = null;
    return; }
  catch(Exception h){ 
    System.out.println(name+" gestures errors: " + h );
    avdeck.drone_av_finished( name, false );
    avatar = null;
    return; }
 if(bTest){
  try{
  int b = (((EventOutMFString)EOfield).getValue()).length;
  int d = b;
   String all [] = new String[b];
   all = (((EventOutMFString)EOfield).getValue());
     for (int c = 0; c<b; c += 1){ 
  if( all[c].equals("NG_no_lod")){ d -= 1; }
 if( all[c].equals("NG_chat_text")){talk = all[c]; d -= 1; }
 if( all[c].equals("NG_humanoid")){human = all[c]; d -= 1; }
   }
 avdeck.addBehavs(all);
  gestures = new EventInSFBool [d];
	}
catch(InvalidEventOutException e){
     System.out.println("UserAvatar gesture string error: " + e);
     avdeck.drone_av_finished( name, false ); 
	 avatar = null;
    return;
	}
	catch(NullPointerException f){ 
    System.out.println("UserAvatar gesture string error: " + f );
    avdeck.drone_av_finished( name, false );
    avatar = null;
    return; }
 catch(InvalidVrmlException g){
     System.out.println("UserAvatar gesture string error: " + g );
     avdeck.drone_av_finished( name, false ); 
	 avatar = null;
    return;
	}
   catch(Exception h){
     System.out.println("UserAvatar gesture string error: " + h );
     avdeck.drone_av_finished( name, false ); 
	 avatar = null;
    return;
	}
 for( int j = 0; j<gestures.length; j += 1 ){
    try {
gestures[j] = (EventInSFBool)nodes[0].getEventIn("gesture" + (j+1));
    }
  catch(NullPointerException f){
    System.out.println(name+" error getting gesture" + (j+1) +": " + f);
    avdeck.drone_av_finished( name, false );
     avatar = null;
    return;
	}
  catch(InvalidEventInException e){
   System.out.println(name+" error getting gesture" + (j+1) +": " + e);
    avdeck.drone_av_finished( name, false );
    avatar = null;
    return;
	}
   catch(InvalidVrmlException h){
    System.out.println(name+" error getting gesture" + (j+1) +": " + h);
    avdeck.drone_av_finished( name, false );
     avatar = null;
    return;
	}
   catch(Exception k){
    System.out.println(name+" error getting gesture" + (j+1) +": " + k);
    avdeck.drone_av_finished( name, false );
     avatar = null;
    return;
	}
  if(j==gestures.length-1){ bGest = true; }
   }
  }
  try{
 EIfield = null;
 bTest = false; 
 EIfield = nodes[i].getEventIn("talker_text");
if(EIfield!=null&&EIfield.getType() == BaseField.MFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
System.out.println( "UserAvatar talker_text error: " + e);
    avdeck.drone_av_finished( name, false );
	 avatar = null;
    return;
}
catch(InvalidEventInException f){
System.out.println( "UserAvatar doesn't have talker_text.");
}
catch(InvalidVrmlException j){
 System.out.println( "UserAvatar talker_text error: " + j);
    avdeck.drone_av_finished( name, false );
	 avatar = null;
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar talker_text error: " + h);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
 if(bTest){
  try { 
  yak = (EventInMFString)EIfield;
   bChat = true;
   name_text = "Group {} "; 
    }
  catch(InvalidEventInException e){ 
  System.out.println("UserAvatar talker_text error: " + e);
  avdeck.drone_av_finished( name, false );
  avatar = null;
  return;
	}
	catch(NullPointerException f){
     System.out.println("UserAvatar talker_text error: " + f);
     avdeck.drone_av_finished( name, false );
    avatar = null;
     return;
	}
  catch(InvalidVrmlException j){
 System.out.println( "UserAvatar talker_text error: " + j);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar talker_text error: " + h);
    avdeck.drone_av_finished( name, false ); 
   avatar = null;
    return;
  }
if(bChat){avdeck.chatDisplay( true );}
 }
   try{
 EIfield = null;
 bTest = false; 
 EIfield = nodes[i].getEventIn("set_name");
if(EIfield!=null&&EIfield.getType() == BaseField.SFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
  System.out.println( "UserAvatar set_name error: " + e);
    avdeck.drone_av_finished( name, false );
    avatar = null;
    return;
}
catch(InvalidEventInException f){
System.out.println( "UserAvatar doesn't have set_name.");
}
catch(InvalidVrmlException j){
 System.out.println( "UserAvatar set_name error: " + j);
    avdeck.drone_av_finished( name, false );
    avatar = null;
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar set_name error: " + h);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
 if(bTest){
 try {  
 set_name = (EventInSFString)EIfield;
   }
		catch(InvalidEventInException e){
     System.out.println("UserAvatar set_name error: " + e);
     avdeck.drone_av_finished( name, false );
	 avatar = null;
	  return;
	   }
	catch(NullPointerException f){
    System.out.println("UserAvatar set_name error: " + f);
    avdeck.drone_av_finished( name, false );   
	 avatar = null;
     return;
	    }
   catch(InvalidVrmlException j){
 System.out.println( "UserAvatar set_name error: " + j);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar set_name error: " + h);
    avdeck.drone_av_finished( name, false ); 
	avatar = null;
    return;
 }
   }
 EventOut EOfield2 = null;
 try{
 EOfield = null;
 bTest = false; 
 EOfield = nodes[i].getEventOut("inMotionBehaviour");
 EOfield2 = nodes[i].getEventOut("notInMotionBehaviour");
if((EOfield!=null&&EOfield.getType() == BaseField.SFString )&&(EOfield2!=null&&EOfield.getType() == BaseField.SFString )){
  bTest = true;
   }
 }
   catch (InvalidEventOutException d ){
   System.out.println("UserAvatar  doesn't have inMotionBehaviour or notInMotionBehaviour! " );
   }
   catch (NullPointerException f ){
    System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + f ); 
   inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
   avatar = null;
    return;
   }
   	    catch (InvalidVrmlException j) {
    System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + j ); 
   inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
   avatar = null;
    return;
	    }	
   catch (Exception ex) {
  System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + ex ); 
  inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
  avatar = null;
  return;
	    }	

 if(bTest){
    try {
		inMotionBehaviour = ((EventOutSFString)EOfield).getValue();
		notInMotionBehaviour = ((EventOutSFString)EOfield2).getValue();
		EventInSFBool iMB = (EventInSFBool)nodes[i].getEventIn(inMotionBehaviour);
        EventInSFBool nIMB = (EventInSFBool)nodes[i].getEventIn(notInMotionBehaviour);
	    }
        catch (InvalidEventOutException d ){
   System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + d );
  inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
   avatar = null;
  return;
   }
   catch (NullPointerException f ){
    System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + f );
  inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
   avatar = null;
  return;
   }
   	    catch (InvalidVrmlException j) {
    System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + j ); 
   inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
  avatar = null;
    return;
	    }	
   catch (Exception ex) {
  System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + ex ); 
  inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
  avatar = null;
  return;
	    }
 try{ 
//setProxy = (EventInSFBool)nodes[i].getEventIn("set_HUD_anim");
EventInSFVec3f setPosition = (EventInSFVec3f)nodes[i].getEventIn("set_position");
EventInSFRotation setOrientation = (EventInSFRotation)nodes[i].getEventIn("set_orientation");
 }
catch(NullPointerException e){
    System.out.println( "A mandatory eventIn 'set_position' or 'set_orientation' is either non existent or incorrect code! " + e);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(InvalidEventInException f){
  System.out.println(" A mandatory eventIn 'set_position' or 'set_orientation' or 'set_UserAvatar' is either non existent or incorrect code! "+ f);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
 catch(InvalidVrmlException g){
    System.out.println(" A mandatory eventIn 'set_position' or 'set_orientation' or 'set_UserAvatar' is either non existent or incorrect code! " + g);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(Exception h){
  System.out.println("A mandatory eventIn 'set_position' or 'set_orientation' or 'set_UserAvatar' is either non existent or incorrect code! "+ h);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
  boolean bDMTest = false;
 try{
EOfield = null;
bTest = false;
EOfield = (EventOutSFBool)nodes[i].getEventOut("dmAvatarStudio");
if(EOfield!=null&&EOfield.getType() == BaseField.SFBool ){
bDMTest = true;
  }
 }
 catch(NullPointerException e){
   System.out.println( "UserAvatar dmAvatarStudio error: " + e );
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(InvalidEventOutException f){
  System.out.println(name + ": is not a AvatarStudio Avatar");
 }
  catch(InvalidVrmlException j){
 System.out.println( "UserAvatar dmAvatarStudio error: " + j);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar dmAvatarStudio error: " + h);
    avdeck.drone_av_finished( name, false ); 
    avatar = null;
    return;
  } 
if(bDMTest){
	try {
eosfb = null;
eosfb = (EventOutSFBool)EOfield;
if(eosfb!=null){ 
if(eosfb.getValue()){ bdmStudioAvatar = true; }
   }
  }
catch(NullPointerException e){
  System.out.println( "UserAvatar dmAvatarStudio error: " + e);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(InvalidEventOutException f){
   System.out.println( "UserAvatar dmAvatarStudio error: " + f);
    avdeck.drone_av_finished( name, false );
   avatar = null;
    return;
 }
  catch(InvalidVrmlException j){
 System.out.println( "UserAvatar dmAvatarStudio error: " + j);
    avdeck.drone_av_finished( name, false );
    avatar = null;
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar dmAvatarStudio error: " + h);
    avdeck.drone_av_finished( name, false );  
	avatar = null;
    return;
     }
    }
   }
 if(bTest||bGest){
try{ 
EventInSFInt32 setProxy = (EventInSFInt32)nodes[i].getEventIn("set_proxy");
setUserAvatar = (EventInSFBool)nodes[i].getEventIn("set_UserAvatar");
}
catch(NullPointerException e){
    System.out.println(name + ": eventIn set_proxy is either non existent or incorrect code! " + e);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidEventInException f){
  System.out.println(name + ": eventIn set_proxy is either non existent or incorrect code! "+ f);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidVrmlException g){ 
System.out.println(name + ": eventIn set_proxy is either non existent or incorrect code! " + g);
     avdeck.drone_av_finished( name, false );
    return;
}
catch(Exception h){
System.out.println(name + ": eventIn set_proxy is either non existent or incorrect code! "+ h);
    avdeck.drone_av_finished( name, false );
    return;
   } 
 }


  }
 else {
 avdeck.drone_av_finished( name, false );
 return;
   }
  }
 }
else 
break;
 }
Node [] help3 = browser.createVrmlFromString( name_text );
try {
  vp = xform[1];
 if(vp.getType().equals("Viewpoint")){
  fov = (EventInSFFloat)vp.getEventIn("set_fieldOfView");
 bind2 = (EventInSFBool)vp.getEventIn("set_bind");
 vppos = (EventInSFVec3f)vp.getEventIn("set_position");
	vpori = (EventInSFRotation)vp.getEventIn("set_orientation");
 System.out.println( "Viewpoint got!" );
}
else { System.out.println( "vp is not Viewpoint Node: " + vp.getType() ); 
  avdeck.drone_av_finished( name, false );
 }
}
catch(InvalidNodeException ine ){ System.out.println( "UA viewpoint node: " + ine );
avdeck.drone_av_finished( name, false );  
return;}
catch(NullPointerException np){ System.out.println( "UA no viewpoint node: " + np );
avdeck.drone_av_finished( name, false ); 
return;}
catch(InvalidVrmlException g ){ System.out.println( "UA viewpoint node: " + g );
avdeck.drone_av_finished( name, false );  
return;}
catch( Exception h){ System.out.println( "UA no viewpoint node: " + h );
avdeck.drone_av_finished( name, false ); 
return;} 
try {
 ((EventInMFNode)xform[0].getEventIn("name_tag")).setValue( help3 );
 which = (EventInSFInt32)xform[0].getEventIn("which");
	// get events
	pos = (EventInSFVec3f)xform[0].getEventIn("translation");
	rot = (EventInSFRotation)xform[0].getEventIn("rotation");
  anchor = xform[0];
 which.setValue(-1);
 }
catch(InvalidNodeException ine ){ System.out.println("UserAvatar get events error: " + ine );
avdeck.drone_av_finished( name, false );  
return;}
catch(NullPointerException np){ System.out.println("UserAvatar get events error: " + np );
avdeck.drone_av_finished( name, false ); 
return;}
catch(InvalidVrmlException g ){ System.out.println("UserAvatar get events error: " + g );
avdeck.drone_av_finished( name, false );  
return;}
catch( Exception h){ System.out.println("UserAvatar get events error: " + h );
avdeck.drone_av_finished( name, false ); 
return;} 
if(bdmStudioAvatar){
ropo.setOffset( new SFVec3f( 0.0f, -0.65f, 0.0f ) );
}
// call back to WorldObject to reset the current Viewpoint in the WorldObject.
 //avdeck.setViewpoint();
 posVal = ropo.positionReturn();
 float [] ft5 = posVal.getValue();
  if(bdmStudioAvatar){ 
 ft5[1] = (ft5[1] + 1.1f); 
		 }
SFVec3f posVal4 = new SFVec3f( ft5 );
pos.setValue(posVal4.getValue());
ropo.set_posChanged( false ); 
rotVal = ropo.orientationReturn();
rot.setValue(rotVal.getValue());
ropo.set_rotChanged( false );
postOffice.handleUserPosition( ft5 );
 runner = new Thread( this, "UserAvatar" );
if(runner!=null){ runner.start(); }
  }
else { 
avdeck.drone_av_finished( name, false );
return; 
  } 
}

    /**
     * shows the Avatars geometrey and binds the Ghost Viewpoint.
     */

    public void show(){
 if(setUserAvatar!=null){ setUserAvatar.setValue( true ); }
 //float [] ft = posVal.getValue();
 //float [] ft2 = rotVal.getValue();
 which.setValue(0);
 posVal2 = ropo.positionGhost();
 rotVal2 = new SFRotation(ropo.orientationReturn());
 beamtostate = false; 
	ropo.show = true;
 bind.setValue(true);	
 //ghostVpPos.setValue( ghostset.plus( posVal2 ).getValue() );
 //ghostVpRot.setValue( rotVal2.getValue() );
    }

    /**
     * reverses the action of show.
     * New 11-12-05 hide method will now set the user's avatar's location to
     * where the user left it when going into OBE or Ghost mode. Previous versions
     * put the user back at the world entry viewpoint.
     */

    public void hide(){  
//System.out.println(" UA hide1 "); 
if(setUserAvatar!=null){ setUserAvatar.setValue( false ); }                            
  ropo.show = false;
 which.setValue(-1);
if(vn!=3){
 vppos.setValue(posVal2.getValue());
 vpori.setValue(rotVal2.getValue());
bind.setValue(false);
if(beamtostate==false){	bind2.setValue(true); } else { bind.setValue(false); }}
    }

     public void beamover( float[] beamdata ){
     float [] ft = { beamdata[0],beamdata[1]+Zaxis,beamdata[2],beamdata[3],beamdata[4],beamdata[5],beamdata[6] };
 //  this.hide(); 
	 ropo.show = false;
	which.setValue(-1);
 //  posVal2 =  new SFVec3f(beamdata[0],beamdata[1],beamdata[2]);
 //  rotVal2 = new SFRotation(beamdata[3],beamdata[4],beamdata[5],beamdata[6]);
   mffloat.setValue(ft);
   avdeck.beamtoFinished();
}
    /**
     * sets correct offset from NavigationInfo so that Avatar at others clients do the
     * right thing.
     */

    public void setOffset( float off ){
	offset = new SFVec3f( 0, -off, 0);
    }

    public String getName(){
	return name;
    }

	   public void chatDisplay ( String [] foo ){
 if(yak!=null){ yak.setValue(foo); }
    }

    public void handleMessage( String tag, Field data ) throws ConsumerException {
	if(!tag.equals(name))
	    throw new ConsumerException(name+" matches not "+tag);
	try {
	    switch(data.getType()){
	    case Field.SFVEC3F:
		pos.setValue(((SFVec3f)data).getValue());
		break;
	    case Field.SFROTATION:
		rot.setValue(((SFRotation)data).getValue());
		break;
	    case Field.SFSTRING:
		if( avatar == null ) break;
		try {
		    EventInSFBool trig = (EventInSFBool)avatar.getEventIn(((SFString)data).getValue());
		    trig.setValue( true );
		}
		catch(Exception e) {
		    System.err.println(name+" err setting "+((SFString)data).getValue()+"\n\t"+e);
		}
		break;
		case Field.SFINT32:
		if(avatar == null ) break;
		try {
		    int iGest = ((SFInt32)data).getValue();
            if(0<gestures.length){
            if(0<=iGest&&0<gestures.length){
		    gestures[iGest].setValue( true );
             }
           }
		}
		catch(Exception e){
		    System.err.println(name+" err setting "+((SFInt32)data).getValue()+"\n\t"+e);
		 }
		break;
	    default:
		throw new ConsumerException("Wrong field type for "+name);
	    }
	}
	catch(IllegalArgumentException e){System.err.println("Wrong type in "+name+"\n\t"+e);}
    }

   public void update(){
	try {
	    if( posVal != null ){
        float [] ft2 = posVal.getValue(); 
       if(bdmStudioAvatar){ 
		 ft2[1] = (ft2[1] + .34f);
		 }
		SFVec3f posVal3 =  new SFVec3f( ft2 );
		float [] ft = posVal3.getValue();
 //System.out.println("UA update3 position: " + ft[0] + " " + ft[1] + " " + ft[2] ); 
postOffice.sendRealTime( RealTimeMessage.NONE, name, posVal3, false );
		ropo.set_posChanged( false );
	
	    }
	    if( rotVal != null ){
		postOffice.sendRealTime( RealTimeMessage.NONE, name, rotVal, false );
	    ropo.set_rotChanged( false );
	    }

	}
	catch(IOException e){ System.err.println("IO in "+name+".update\n\t"+e); }
	if(!moving&&notInMotionBehaviour!=null){ 
     this.trigger( notInMotionBehaviour );
	  }
    } 
   
    public void stop(){
    runner = null;
    } 

    public void delete() throws RuntimeException {
	runner = null;
	ropo.dead = true;
	ropo.delete();
   if(WSC!=null){ WSC.delete(); }
   if(WSLC!=null){ WSLC.delete(); }
Enumeration list = states.elements();
	while( list.hasMoreElements()){
	    postOffice.removeConsumer(((Route)list.nextElement()).getName());
	}  
  yak = null;vp = proxy = anchor = avatar = null; 
    }

 /**
  * New 11-12-05 Here to satisfy need of interface
  */

  public void beamto( String target ){
 if( target.equals(name) ){
   posflt = ropo.positionReturn().getValue(); rotflt = ropo.orientationReturn().getValue();
   beam[0] = posflt[0]; beam[1] = posflt[1]; beam[2] = posflt[2];
   beam[3] = rotflt[0]; beam[4] = rotflt[1]; beam[5] = rotflt[2]; beam[6] = rotflt[3];
   avdeck.beamtocoord(beam);
  }
}
  
 public float [] getUserPosition_Rotation(){
   posflt = ropo.positionReturn().getValue(); rotflt = ropo.orientationReturn().getValue();
   beam[0] = posflt[0]; beam[1] = posflt[1]; beam[2] = posflt[2];
   beam[3] = rotflt[0]; beam[4] = rotflt[1]; beam[5] = rotflt[2]; beam[6] = rotflt[3];
   return beam;
 }

    public void userView( String target ){
 }

  public boolean getbAvatar(){
  return bAvatar;
     }

  public void setUserPosition( float[] flt ){
  }

    public void checkPosition( float[] fUP, boolean bUPs  ){
 }
  
   public void objectRoutes( String target ){
 // if(name.endsWith("_Obj")){
// Node PM = avdeck.getPrimeMover();
// Browser browser = avdeck.getBrowser();
//  if(bOR==true){
//   ((EventInSFBool)proxy.getEventIn("enabled")).setValue( false );
//   ropo.set_posChanged( false );
//   ropo.set_rotChanged( false );
//   browser.addRoute( PM, "orientation_changed", anchor, "rotation");
//   browser.addRoute( PM, "position_changed", anchor, "translation" );
//  bOR = false;
//  }
//  else {
//   browser.deleteRoute( PM, "orientation_changed", anchor, "rotation" );
//   browser.deleteRoute( PM, "position_changed", anchor, "translation" );
// ((EventInSFBool)proxy.getEventIn("enabled")).setValue( true );
//   bOR = true;
//   }
//  }
 }

 /**
 * New 11-12-05 When run starts it tells Beta (avdeck) that the UserAvatar is done
 * by calling "user_av_finished" If the user's avatar is optioned for chat text display,
 * it will send the EventInMFString yak to Beta.
 * A big change is the sleep time increased from 100 to 400 milliseconds. Much easier on the network!
 *
 */
    public void run(){
   if(bChat&&set_name!=null){
  set_name.setValue(name.substring(1));
   }
avdeck.user_av_finished();
	while( Thread.currentThread() == runner ) {
	    try { 
if(!ropo.bProxy){
	  moving = false;
      boolean done = false; 
		if( ropo.get_posChanged() ){
       moving = true;
      posVal = ropo.positionReturn();
      postOffice.handleUserPosition( posVal.getValue() );
		    postOffice.sendRealTime( RealTimeMessage.NONE, name, posVal, false ); 
         pos.setValue(posVal.getValue());
		  ropo.set_posChanged( false ); 
		}
		if( ropo.get_rotChanged() ){
      moving = true;
      rotVal = ropo.orientationReturn();
   postOffice.sendRealTime( RealTimeMessage.NONE, name, rotVal, false );
   rot.setValue(rotVal.getValue());
		  ropo.set_rotChanged( false ); 
		 }
       }
		// end
		Thread.sleep(lSleep);
	    }
	    catch(InterruptedException e){}
	    catch(IOException e){
		System.err.println("IOEx in "+name+".run\n\t"+e);
	    }
	}
    }

	public void gesture( int gest ){
	try {
	    postOffice.sendRealTime( RealTimeMessage.NONE, name, new SFInt32( gest ), true );
	}
	catch(IOException e) {
	    System.err.println(name+" error posting "+gest+"\n\t"+e);
	 }
    }

    /**
     * posts a behaviour event.
     *
     * @param behab the name of the behaviour
     */
    private void trigger( String behab ){
	try {
	    postOffice.sendRealTime( RealTimeMessage.NONE, name, new SFString( behab ), false );
	}
	catch(IOException e) {
	    System.err.println(name+" error posting "+behab+"\n\t"+e);
	}
    }
}
