/*
  WorldSendChat.java

  Copyright (C) 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */









package matrix.deck.eai2;

import matrix.vrml.*;
import matrix.deck.*;
import java.io.*;
import java.util.*;
import vrml.eai.*;
import vrml.eai.field.*;
import vrml.eai.event.*;

/**
 * New 11-12-05 WorldSendChat sends chat messages from world to network chat via the Beta class.
 * Instanced by WorldObject class.
 * @version 0.1    
 * @author Russ Kinter
 */

public class WorldSendChat implements VrmlEventListener, Runnable {
    /**
     *  New 11-12-05 chat holds the eventOut of Node wsc ( WorldSendChat ) called "to_chat" 
     *  It is given an EventObserver to watch for new Strings
     */
  protected EventOutMFString chat;
      /**
     *  New 11-12-05 Beta deck to which the Strings are sent for output over the network
     */
  protected Beta deck;
     /**
     *  New 11-12-05 wsc holds the instance of the Proto WorldSendChat
     */
  protected Node wsc;
    /**
     *  New 11-12-05 runner controls the rate of how many strings go to Beta deck and the network.
     */
  protected Thread runner = null;
    /**
     *  New 11-12-05 Vector vec is a buffer storing strings before the Thread runner sends them.
     */
  protected Vector vec = new Vector();


  public  WorldSendChat(Beta beta, Node WSC ){  
  deck = beta; 
  wsc = WSC; 
  chat = (EventOutMFString)wsc.getEventOut("to_chat");
  chat.addVrmlEventListener(this);
  runner = new Thread( this,"Yak_To_Chat" );
  runner.start();
}

  public  WorldSendChat(Beta beta, EventOutMFString eomfs ){  
  deck = beta; 
  chat = eomfs;
  chat.addVrmlEventListener(this);
  runner = new Thread( this,"Yak_To_Chat" );
  runner.start();
  }



  public WorldSendChat(){
}

  protected void holdChat() throws InterruptedException {
    synchronized(vec) { 
     vec.wait();
    }
   }

   /**
     *  New 11-12-05 EventOut Observer that loads new incoming Strings from the VRML World into the Vector "vec"
     *  which in turn wakes up the Thread Runner to send to Beta Deck in 1 second interegals
     */



 public void eventOutChanged(VrmlEvent vrmlevent ){
	  int b = 0;
	  BaseField bf = vrmlevent.getSource();
    try {
	    if( bf.getType()!= BaseField.MFString ) {
		System.err.println("Got wrong value from ");
		return;
	    }
     b = chat.getValue().length;
     String[] st = new String[b];
     st = chat.getValue();
       synchronized(vec) { 
     for (int c = 0; c<b; c++){ 
      vec.insertElementAt(st[c],c); } 
       vec.notify();
       }
	    }
	catch(Exception e) {
	    System.err.println("Error converting msg from world"+"\n\t" +e);
	}
    }

  /**
     *  New 11-12-05 sendChat is called by run once a second ( wait(1000) if the Vector "vec" still
     *  contains Strings. If it doesn't, it goes to sleep ( wait() ) till waken up by callback - the eventOut observer 
     */

 protected void sendChat() throws InterruptedException {
    synchronized(vec) { 
       if(!vec.isEmpty()){
       String st2 = vec.firstElement().toString();
       deck.handleChat2( st2, null );
       vec.removeElementAt(0);
       vec.wait(1000);
        }
    else { 
      vec.wait();
      }
    }
  }
  
 /**
     *  New 11-12-05 When run first starts it goes into wait by calling holdChat
     */

 public void run(){
    try { 
     holdChat();
      }
    catch(InterruptedException e){}
  while( Thread.currentThread() == runner ) {
 try {
    sendChat();
   } 
 catch(InterruptedException e){}
 
  }
}

    /**
     *  New 11-12-05 clean up at either disconnect or room change. 
     *  Notifying the Vector being used as a buffer at clean up was used in the original
     *  DeepMatrix (Connection inner class MessageBuffer ) and therefore adopted here as well. 
     */

  public void delete(){
  if(chat!=null){ chat.removeVrmlEventListener(this);}
   synchronized(vec){ vec.removeAllElements(); vec.notify() ;}
   runner = null; chat = null; wsc = null;
   } 
}