/*
 Log.java
 by Frederik Schorr
 with changes by Geometrek
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrixIP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package matrix.server;

import java.io.*;
import java.text.*;
import java.util.*;

/**
 * Logs messages with timestamps and thread ids to file.
 * This class was developed by Frederik Schorr during a CS course.
 * I added some minor changes.
 */
public class Log {
	PrintWriter m_writer = null;
	Date date = new Date();
	DateFormat df;
	
	final int BUFFERSIZE = 4096;
	
	/**
	 * Open log file or if impossible use err stream.
	 */
	public Log(String fileName) {
		df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM );
		try {
			if( fileName == null || fileName.equals("-"))
				m_writer = new PrintWriter( new BufferedOutputStream( System.out, BUFFERSIZE ));
			else
				m_writer = new PrintWriter( new BufferedOutputStream( new FileOutputStream(fileName, true) , BUFFERSIZE ));
			m_writer.println("------------------------ New session ------------------------");
			m_writer.println(df.format(date)+" logging started into file "+fileName);
			m_writer.flush();
		}
		catch(Exception e) {
			System.err.println("Could not open file " + fileName + " for logging");
			m_writer = new PrintWriter( System.err );
		}
	}
	
	/**
	 * Close log file
	 */
	public synchronized void close() {
		try {
			m_writer.flush();
			m_writer.close();
		}
		catch(Exception e) {
			System.err.println("Error closing log file");
		}
	}
	
	/**
	 * Write timestamp, threadid & message to log file.
	 */
	public synchronized String println(String msg) {
		try {
			date.setTime( System.currentTimeMillis());
			m_writer.println(df.format(date) + " " + Thread.currentThread().getName() + " : " + msg);	    
			m_writer.flush();	    
		}
		catch(Exception e) {}
		return msg;
	}
}
