/**
  MatrixServlet.java

  Copyright (C) 2001 Geometrek
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.server;

import java.io.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * This servlet enables HTTP tunneling to the Matrix server. It runs a
 * matrix server internally, which can also be accessed via the
 * usuall TCP connection. Therefore no standalone Matrix server is needed.
 * @author  gr
 */
public class MatrixServlet extends HttpServlet {

    // holds the actual server
    protected MatrixD daemon;
       protected HttpSession session = null;
       protected HttpServletConnection connection = null;
       protected ServerThread st = null;
    /** Initializes the servlet.
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        daemon = (MatrixD) getServletContext().getAttribute("deepmatrix.server");
        if( daemon == null ){
            String rcfile = config.getInitParameter("rcfile");
            if( rcfile != null ){
                File path = new File( rcfile );
                if( !path.isAbsolute() ){
                    rcfile = getServletContext().getRealPath( rcfile );
                }
                getServletContext().log("rcfile "+rcfile);
            }
            int port = MatrixD.PORT;
            if( config.getInitParameter("port") != null ){
                try {
                    port = Integer.parseInt( config.getInitParameter("port"));
                }
                catch( NumberFormatException e){
                    getServletContext().log( "Error converting port number "+config.getInitParameter("port"),e);
                    port = MatrixD.PORT;
                }
            }
            int loglevel = 0;
            if( config.getInitParameter("loglevel") != null ){
                try {
                    loglevel = Integer.parseInt( config.getInitParameter("loglevel"));
                    if( loglevel < 0 || loglevel > 3 )
                        loglevel = 0;
                }
                catch( NumberFormatException e){
                    getServletContext().log( "Error converting loglevel number "+config.getInitParameter("loglevel"),e);
                    loglevel = 0;
                }
            }
            daemon = new MatrixD( rcfile, port, loglevel );
            getServletContext().setAttribute( "deepmatrix.server", daemon );
            getServletContext().log("Started DeepMatrix Daemon");
        }
    }

    /** Destroys the servlet.
     */
    public void destroy() {
        daemon.destroy();
        super.destroy();
        getServletContext().log("Shutting down DeepMatrix Daemon");
    }

    /**
     * Method to Service the input requests. The input Requests are redirected to
     * the Remote Server. The output from the Remote Server is sent back as the
     * response.
     **/
        public void service(HttpServletRequest req, HttpServletResponse res) 
            throws ServletException, IOException {
        // get session and check for a new connection
        HttpSession session = req.getSession(true);
        HttpServletConnection connection = (HttpServletConnection)session.getAttribute("deepmatrix.connection");
        if( connection == null ){  // oh, we have a new client !
            connection = new HttpServletConnection(InetAddress.getByName(req.getRemoteHost()));
            session.setAttribute("deepmatrix.connection", connection );
            ServerThread user = new ServerThread( daemon, daemon.startRoom, connection );            
            try {
                Thread.sleep(200);
            }
            catch(Exception e) {}
            getServletContext().log("New connection from "+req.getRemoteHost());
        } else {        
            // ups connection is not alive anymore -> start disconnection counter and
            // destroy
            if( !connection.isAlive() ){                
                Integer count = (Integer)session.getAttribute("deepmatrix.disconnect");
                if( count == null ){                   
                    session.setAttribute("deepmatrix.disconnect", new Integer(1) );
                } else {
                    int val = count.intValue();
                    val--;
                    if( val == 0 ){                        
                        session.removeAttribute("deepmatrix.connection");
                        session.invalidate();
                        getServletContext().log("Connection from "+req.getRemoteHost()+" closed");
                    } else {
                        session.setAttribute("deepmatrix.disconnect", new Integer( val ));
                    }
                }
            }    
        }
        
        // Get the Object Input Stream from the Request
        ObjectInputStream in = new ObjectInputStream(req.getInputStream());
        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
        ObjectOutputStream out = new ObjectOutputStream(byteOut);
        
        // Pass the request to HttpConnection for processing.
        try {
            connection.exchange( in, out ); 
        }
        catch( Exception e ){
            getServletContext().log("Error exchanging messages ", e);
            daemon.log("MatrixServlet : Error exchanging messages "+e,1);
        }
           
        out.flush();
        byte[] buf = byteOut.toByteArray();
        res.setContentLength(buf.length); 
        ServletOutputStream servletOut = res.getOutputStream();
        servletOut.write(buf); 
        in.close();
        out.close();
    }

    /** Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Enables HTTP tunneling to the DeepMatrix server";
    }
}
