/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

public class FieldInputStream extends DataInputStream {

  public FieldInputStream(InputStream in) {
    super(in);
  }

  public Field readField() throws IOException {
    byte type = readByte();
    switch(type+1) {
    case Field.NOTHING:
      return null;
    case Field.SFBOOL:
      return new SFBool(this);
    case Field.SFVEC3F:
      return new SFVec3f(this);
    case Field.SFROTATION:
      return new SFRotation(this);
    case Field.SFSTRING:
      return new SFString(this);
    case Field.MFSTRING:
      return new MFString(this);
    case Field.SFCOLOR:
      return new SFColor(this);
    case Field.SFFLOAT:
      return new SFFloat(this);
    case Field.SFIMAGE:
      return new SFImage(this);
    case Field.SFINT32:
      return new SFInt32(this);
    case Field.SFTIME:
      return new SFTime(this);
    case Field.SFVEC2F:
      return new SFVec2f(this);
    case Field.MFCOLOR:
      return new MFColor(this);
    case Field.MFFLOAT:
      return new MFFloat(this);
    case Field.MFINT32:
      return new MFInt32(this);
    case Field.MFROTATION:
      return new MFRotation(this);
    case Field.MFVEC2F:
      return new MFVec2f(this);
    case Field.MFVEC3F:
      return new MFVec3f(this);
    case Field.FIELDARRAY:
      return new FieldArray(this);
    default :
      throw new UnsupportedFieldTypeException("type " + type);
    }
  }
}

