/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for MFRotation field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class MFRotation extends Field implements Serializable {
  SFRotation[] values;

  public MFRotation( float[][] val ){
    setValue( val );
  }

  public MFRotation( MFRotation val ){
    setValue( val.getValue());
  }

  public MFRotation(DataInputStream in) throws IOException {
    values = new SFRotation[in.readInt()];
    for (int i = 0; i < values.length; i++) {
      values[i] = new SFRotation(in);
    }
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeInt(values.length);
    for (int i = 0; i < values.length; i++) {
      values[i].write(out);
    }
  }

  public byte getType() { return MFROTATION; }
  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != MFROTATION )
      throw new MatrixInvalidFieldException("Data not MFRotation field");
    values = new SFRotation[((MFRotation)data).values.length];
    for (int i = 0; i < values.length; i++) {
      values[i] = new SFRotation(((MFRotation)data).values[i] );
    }
  }

  public void setValue( float[][] data ){
    values = new SFRotation[data.length];
    for( int i = 0; i < data.length ;i++ ) {
      values[i] = new SFRotation( data[i] );
    }
  }

  public SFRotation[] getRotations() {
    return values;
  }

  public SFRotation get1Rotation( int pos ) {
    return values[pos];
  }

  public float[][] getValue() {
    float[][] out = new float[values.length][4];
    for( int i = 0 ; i < values.length ; i ++ ){
      out[i] = values[i].getValue();
    }
    return out;
  }

  public float[] get1Value( int pos ){
    return values[pos].getValue();
  }
}
