/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for SFColor field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class SFColor extends Field implements Serializable {
  public float red, green, blue;

  public SFColor( float[] data ){
    setValue( data );
  }

  public SFColor( SFColor data ) {
    red = data.red;
    green = data.green;
    blue = data.blue;
  }

  public SFColor(DataInputStream in) throws IOException {
    red = in.readFloat();
    green = in.readFloat();
    blue = in.readFloat();
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeFloat(red);
    out.writeFloat(green);
    out.writeFloat(blue);
  }

  public byte getType() { return SFCOLOR; }

  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != SFCOLOR )
      throw new MatrixInvalidFieldException("Data not SFColor field");
    red = ((SFColor)data).red;
    green = ((SFColor)data).green;
    blue = ((SFColor)data).blue;
  }

  public void setValue( float[] data ){
    red = data[0];
    green = data[1];
    blue = data[2];
  }

  public float[] getValue(){
    float[] out = new float[3];
    out[0] = red;
    out[1] = green;
    out[2] = blue;
    return out;
  }
}
