/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for SFVec2f field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class SFVec2f extends Field implements Serializable {
  public float x, y;

  public SFVec2f( SFVec2f data ) {
    setValue( data.getValue());
  }

  public SFVec2f( float[] val ){
    setValue( val );
  }

  public SFVec2f(float x, float y) {
    this.x = x;
    this.y = y;
  }

  public SFVec2f(DataInputStream in) throws IOException {
    x = in.readFloat();
    y = in.readFloat();
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeFloat(x);
    out.writeFloat(y);
  }

  public byte getType() { return SFVEC2F; }

  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != SFVEC2F )
      throw new MatrixInvalidFieldException("Data not SFVec2f field");
    setValue(((SFVec2f)data).getValue());
  }

  public void setValue( float[] val ){
    x = val[0];
    y = val[1];
  }

  public float[] getValue(){
    float[] help = new float[2];
    help[0] = x;
    help[1] = y;
    return help;
  }
}
