/**
 * EAITool.java
 *  This file is part of DeepMatrixIP9.
 *  Copyright (C) 2001 Geometrek 
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrixIp9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml.eai2;

import matrix.vrml.*;
import vrml.eai.*;
import vrml.eai.field.*;


/**
 * Helper class to get and set EventOuts/Ins of the VRML EAI.
 */
public class EAITool {

    private EAITool(){
    }

     public static void setEventIn( EventIn event, Field value ) throws MatrixInvalidFieldException, IllegalArgumentException {
    switch( event.getType()){
      case BaseField.SFBool :
        if( value.getType() != value.SFBOOL )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFBool)event).setValue(((SFBool)value).value);
        break;
      //case BaseField.SFImage :
      //  if( value.getType() != value.SFIMAGE )
      //    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
      //  SFImage im = (SFImage)value;
      //  ((EventInSFImage)event).setValue(im.width, im.height, im.components, im.pixels);
      //  break;
      case BaseField.SFTime :
        if( value.getType() != value.SFTIME )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFTime)event).setValue(((SFTime)value).time);
        break;
      case BaseField.SFColor :
        if( value.getType() != value.SFCOLOR )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFColor)event).setValue(((SFColor)value).getValue());
        break;
      case BaseField.MFColor :
        if( value.getType() != value.MFCOLOR )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInMFColor)event).setValue(((MFColor)value).getValue());
        break;
      case BaseField.SFFloat :
        if( value.getType() != value.SFFLOAT )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFFloat)event).setValue(((SFFloat)value).value);
        break;
      case BaseField.MFFloat :
        if( value.getType() != value.MFFLOAT )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInMFFloat)event).setValue(((MFFloat)value).values);
        break;
      case BaseField.SFInt32 :
        if( value.getType() != value.SFINT32 )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFInt32)event).setValue(((SFInt32)value).value);
        break;
      case BaseField.MFInt32 :
        if( value.getType() != value.MFINT32 )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInMFInt32)event).setValue(((MFInt32)value).values);
        break;
      case BaseField.SFRotation :
        if( value.getType() != value.SFROTATION )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFRotation)event).setValue(((SFRotation)value).getValue());
        break;
      case BaseField.MFRotation :
        if( value.getType() != value.MFROTATION )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInMFRotation)event).setValue(((MFRotation)value).getValue());
        break;
      case BaseField.SFString :
        if( value.getType() != value.SFSTRING )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFString)event).setValue(((SFString)value).s);
        break;
      case BaseField.MFString :
        if( value.getType() != value.MFSTRING )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInMFString)event).setValue(((MFString)value).strings);
        break;
      case BaseField.SFVec2f :
        if( value.getType() != value.SFVEC2F )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFVec2f)event).setValue(((SFVec2f)value).getValue());
        break;
      case BaseField.MFVec2f :
        if( value.getType() != value.MFVEC2F )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInMFVec2f)event).setValue(((MFVec2f)value).getValue());
        break;
      case BaseField.SFVec3f :
        if( value.getType() != value.SFVEC3F )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInSFVec3f)event).setValue(((SFVec3f)value).getValue());
        break;
      case BaseField.MFVec3f :
        if( value.getType() != value.MFVEC3F )
          throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
        ((EventInMFVec3f)event).setValue(((MFVec3f)value).getValue());
        break;
      default:
        throw new MatrixInvalidFieldException("Not supported Field Type");
    }
  }

  public static Field makeField( EventOut event ) throws MatrixInvalidFieldException {
    switch( event.getType()){
      case BaseField.SFBool :
        return new SFBool(((EventOutSFBool)event).getValue());
      //case BaseField.SFImage :
      //  EventOutSFImage im = (EventOutSFImage)event;
      //  return new SFImage(im.getWidth(), im.getHeight(), im.getComponents(), im.getPixels());
      case BaseField.SFTime :
        return new SFTime(((EventOutSFTime)event).getValue());
      case BaseField.SFColor :
        return new SFColor(((EventOutSFColor)event).getValue());
      case BaseField.MFColor :
        return new MFColor(((EventOutMFColor)event).getValue());
      case BaseField.SFFloat :
        return new SFFloat(((EventOutSFFloat)event).getValue());
      case BaseField.MFFloat :
        return new MFFloat(((EventOutMFFloat)event).getValue());
      case BaseField.SFInt32 :
        return new SFInt32(((EventOutSFInt32)event).getValue());
      case BaseField.MFInt32 :
        return new MFInt32(((EventOutMFInt32)event).getValue());
      case BaseField.SFRotation :
        return new SFRotation(((EventOutSFRotation)event).getValue());
      case BaseField.MFRotation :
        return new MFRotation(((EventOutMFRotation)event).getValue());
      case BaseField.SFString :
        return new SFString(((EventOutSFString)event).getValue());
      case BaseField.MFString :
        return new MFString(((EventOutMFString)event).getValue());
      case BaseField.SFVec2f :
        return new SFVec2f(((EventOutSFVec2f)event).getValue());
      case BaseField.MFVec2f :
        return new MFVec2f(((EventOutMFVec2f)event).getValue());
      case BaseField.SFVec3f :
        return new SFVec3f(((EventOutSFVec3f)event).getValue());
      case BaseField.MFVec3f :
        return new MFVec3f(((EventOutMFVec3f)event).getValue());
      default:
        throw new MatrixInvalidFieldException("Not supported Field Type");
    }
  }
    
}
